

/*
	The only reason to use the below, is if you want to allow developers to run ConnectDBCon()
	without specifying arguments, as they will default to using the below. There is no practical
	reason for that.

	This may be removed in the future. A better solution is to define consts that look like these
	in your own game, so that updating this library won't require you to re-type the below.
*/

/* THE FOLLOWING VARIABLES CAN BE CHANGED */
var/const
	_MYSQL_DB = "alathon"
	_MYSQL_ADDR = "localhost:3306"
	_MYSQL_USER = "alathon"
	_MYSQL_PASS = "310388"

	MYSQL_DESCRIBE_DEFAULT = 1 // Setting this to 1 means mysql.AddTable() will query the SQL database for
							   // whether the table exists or not already. If it does not exist, it will
							   // run table.InitInDB() if the table has an init_in_db variable set.
							   // Note that the table will still be added to the tables list, even if the
							   // describe fails and the table doesn't have an init_in_db variable set.

							   // Setting this to 0 means mysql.AddTable() will not care whether the table
							   // exists in the SQL database, and will merely add it to the tables list.
/*************************************************/

var/const
	_MYSQL_DBI = "dbi:mysql:" + _MYSQL_DB + ":" + _MYSQL_ADDR

DBConnection
	var/last_used = 0

	Connect()
		. = ..(arglist(args))
		if(!.)
			world.log << "MySQL Connect() unsuccessful: [ErrorMsg()]"
		else
			world.log << "MySQL Connect() successful"

	NewQuery(locked = 0)
		var/DBQuery/Q = ..()
		if(locked) Q.Lock()
		return Q

DBQuery
	var
		name   = "NO QUERY RUN"
		locked = 0

	proc
		Lock()
		Unlock()

	Lock()
		locked = 1

	Unlock()
		locked = 0

	NextRow()
		db_connection.last_used = world.realtime
		return ..()

	RowsAffected()
		db_connection.last_used = world.realtime
		return ..()

	RowCount()
		db_connection.last_used = world.realtime
		return ..()

	Columns()
		db_connection.last_used = world.realtime
		return ..()

	GetColumnData()
		db_connection.last_used = world.realtime
		return ..()

	Execute(S)
		db_connection.last_used = world.realtime
		name = S
		return ..()

var/mysql/mysql
mysql
	var
		name = "db_name"
		_sql_timeout = 150 // 15 seconds before the SQL connection is closed.
		_ignore_locks = 0
		DBConnection/dbcon
		queries[0]
		tables[0]

	proc
		Load()
		AddTable()
		RemTable()
		GetTable()
		SQL()
		MultiSQL()
		Heartbeat()
		ConnectDBCon()
		DisconnectDBCon()
		IsOld()
		AddQuery()
		RemoveQuery()
		RemoveQueries()
		UnlockQuery()
		LockQuery()
		SetSQLTimeout()

	SetSQLTimeout(n)
		_sql_timeout = max(n,0)
		Heartbeat()

	UnlockQuery(DBQuery/Q)
		if(istype(Q))
			UnlockQuery(Q)
			return 1
		return 0

	LockQuery(DBQuery/Q)
		if(istype(Q))
			LockQuery(Q)
			return 1
		return 0


	Load()
		if(!length(tables))
			for(var/a in typesof(/mysql_table/) - /mysql_table/)
				var/mysql_table/T = new a()
				AddTable(T)
				if(!T.db_name) T.db_name = src.name
			return 1
		return 0

	AddTable(mysql_table/T, describe = MYSQL_DESCRIBE_DEFAULT)
		if(!istype(T)) return 0
		if(!tables) tables = new()
		if(T in tables) return null

		var/table_name = T.Name()

		if(describe)
			// Don't add if the table is 'bad'
			var/DBQuery/Q = AddQuery()
			if(!Q)
				return 0
			var/ok = Q.Execute("DESCRIBE [table_name]")
			if(!ok)
				if(T.init_db_query)
					T.InitInDB()

		tables += table_name
		tables["[table_name]"] = T
		return 1

	RemTable(mysql_table/T)
		if(!istype(T)) return 0
		var/total_name = T.Name()
		if(!tables || !(total_name in tables)) return null
		tables -= total_name
		if(!length(tables)) tables = null
		return 1

	GetTable(n)
		if(!n || !(n in tables)) return null
		return tables[n]

	SQL(a)
		if(!a) return null

		var/DBQuery/Q
		Q = AddQuery()
		if(!istype(Q))
			return 0
		Q.Execute(a)
		return Q

	MultiSQL(fetch = 0)
		var/alen = length(args)
		var/i	 = 1
		var/DBQuery/Q

		if(alen < 2) return 0

		Q = AddQuery()
		if(!istype(Q))
			return 0

		for(i, i <= alen, i++)
			Q.Execute(args[i])
		if(fetch)
			Q.NextRow()
			return Q
		else
			return Q

	IsOld()
		return (world.realtime - dbcon.last_used >= _sql_timeout)

	Heartbeat()
		set background = 1
		if(!_sql_timeout) // No timeout. Indefinate connection.
			return

		spawn(_sql_timeout)
			if(istype(dbcon) && IsOld())
				if(!DisconnectDBCon()) // Failed to disconnect due to locked queries.
					Heartbeat()
			else
				Heartbeat()

	ConnectDBCon(_dbi = _MYSQL_DBI, _user = _MYSQL_USER, _pass = _MYSQL_PASS)
		if(!dbcon)
			dbcon = new()
			dbcon.last_used = world.realtime
			Heartbeat()
			return dbcon.Connect(_dbi, _user, _pass)
		else if(dbcon.IsConnected())
			return -1

	DisconnectDBCon()
		if(dbcon)
			dbcon.Disconnect()
		RemoveQueries()
		if(length(queries))
			return 0
		del dbcon
		return 1

	AddQuery(locked = 0)
		if(!dbcon)
			var/ok = ConnectDBCon()
			if(!ok)
				return 0

		if(!queries) queries = new()
		. = dbcon.NewQuery(locked)
		if(istype(., /DBQuery))
			queries += .
		dbcon.last_used = world.realtime

	RemoveQuery(DBQuery/Q, force = 0)
		if(!queries || !(Q in queries)) return 0
		if(!force && Q.locked && !_ignore_locks) return 0
		queries -= Q
		Q.Close()
		del Q
		if(!length(queries)) queries = null
		return 1

	RemoveQueries(force = 0)
		. = 0
		if(!queries) return
		for(var/DBQuery/Q in queries)
			if(Q.locked && !force && !_ignore_locks) continue
			Q.Close()
			del Q
			. = 1
		if(!length(queries)) queries = null

// Added for ease of use.
datum/proc/FillVariables(list/L)
	. = 0
	if(istype(L, /list) && length(L))
		for(var/a in L)
			if(a in vars)
				L[a] = vars[a]
				. = 1