/*

This is a simple demo showcasing how to set up an example table
and how to save/load something from the database. There are lots of
other uses and possibilities, this is just a short example

*/


// Change the below 4 to match the configuration of your MySQL server/database
var/const
	DB_NAME = "test"
	DB_ADDR = "localhost:3306"
	DB_USER = "john"
	DB_PASS = "doe"

var/const
	DB_DBI  = "dbi:mysql:" + DB_NAME + ":" + DB_ADDR

var/list/built_in_mob = list("id", "type", "tag", "suffix", "overlays", "underlays", "dir", "luminosity", "opacity", "density", "layer", "mouse_over_pointer",\
"mouse_drag_pointer", "mouse_drop_pointer", "mouse_drop_zone", "contents", "invisibility", "infra_luminosity", "pixel_x", "pixel_y",\
"sight", "see_in_dark", "see_invisible", "see_infrared", "pixel_step_size")

mysql_table
	mob
		tbl_name = "mobs"
		variables = null // Auto generate list of variables from /mob subtype
		variables_type = /mob

		IsException(n) // Decline the auto-generated list any variables which match those in built_in_mob.
			return (n in built_in_mob)

		New()
			..()
			// Add the 'last_modified' variable manually, since SQL uses it but DM does not.
			AddVariable("last_modified")

mob
	var
		id			= 0
		hp 			= 10
		mana 		= 10

world/New()
	..()
	mysql = new()
	mysql.name = "alathon"
	mysql.Load()
	var/ok = mysql.ConnectDBCon(DB_DBI, DB_USER, DB_PASS)
	if(!ok)
		world.log << "MySQL initialization failed. Check the world.log for an error message"

mob/Stat()
	if(statpanel("Stats"))
		stat("HP:", hp)
		stat("MANA:", mana)

mob/verb/Sethp(n as num)
	hp = n

mob/verb/Setmana(n as num)
	mana = n

client/verb/Save()
	src << "Saving character to MySQL DB"
	var/mysql_table/T = mysql.GetTable("mobs")
	var/list/to_save = T.variables.Copy()

	mob.FillVariables(to_save)
	to_save["last_modified"] = "[time2text(world.realtime, "YYYY-MM-DD")] [time2text(world.timeofday, "hh:mm:ss")]"

	if(mob.id) // if I have an id, I've been saved before. This is an update
		T.Update(to_save)

	else
		var/DBQuery/Q = T.Insert(L = to_save, fetch = 1) // Fetch = 1 runs "SELECT LAST_INSERT_ID() FROM table_name" right after,
		mob.id = Q.item[1]									 // to get an ID from the last insertion into table.

client/verb/Load()
	var/mysql_table/T = mysql.GetTable("mobs")
	var/DBQuery/Q = T.Select("*") // Select entire table
	if(!istype(Q))
		src << Q
		return

	var/list/array = T.Query2List(Q)
	if(!length(array))
		src << "No mobs saved"
		return

	for(var/i = 1, i <= length(array), i++)
		var/list/data = array[i]
		src << "ID:[data["id"]] NAME:[data["name"]] HP:[data["hp"]] MANA:[data["mana"]]"

	var/id = 1
	while(id)
		id = input(src, "Which character ID do you want to load? (0 to exit)") as num

		if(id > 0) for(var/i = 1, i <= length(array), i++)
			var/list/data = array[i]
			if(data["id"] == id)
				for(var/a in data)
					if(a in mob.vars)
						mob.vars[a] = data[a]
				src << "Character loaded"
				return
	src << "Cancelled"

