#define EXCLUDE_TMP

proc
	convert_char(n, c_from = "_", c_to = " ")
		if(!n || !istext(n)) return

		. = n
		var/find = findtext(., c_from)
		while(find)
			. = copytext(.,1,find) + c_to + copytext(.,find+1)
			find = findtext(., c_from)


mysql_table
	var
		tbl_name = ""
		db_name  = ""
		init_db_query = ""
		variables_type = null
		list
			variables = list()

	proc
		IsException()
		AddVariable()
		IsVariable()
		List2Text()
		DM2MySQL()
		MySQL2DM()
		InitInDB()
		Query2List()
		Name()
		Select()
		Insert()
		Update()
		SelectQuery()
		InsertQuery()
		UpdateQuery()
		Translate()
		mysql2dm()
		dm2mysql()

	New()
		if(isnull(variables) && !isnull(variables_type))
			var/datum/A = new variables_type()
			for(var/v in A.vars)
#ifdef EXCLUDE_TMP
				if(!issaved(A.vars[v])) continue
#endif
				AddVariable(v)
			del A

	IsException(n)
		return 0

	AddVariable(name, datum/O)
		if(name in variables) return 0
		if(!IsException(name))
			variables += name
		return (name in variables)

	// Thanks Dan --
	List2Text(lst[],delim,pad="")
		var/i
		if(!istype(lst,/list)) return lst
		for(i=1, i<=lst.len, i++)
			. = "[ . ][pad][ lst[i] ][pad][ (i+1>lst.len) ? "" : delim]"

	IsVariable(n)
		return (n && variables && n in variables)

	mysql2dm(var_n, var_value)
		if(!IsVariable(var_n)) return 0
		var/var_name = convert_char(var_n)

		if(!hascall(src,"mysql2dm [var_name]"))
			return var_value
		return call(src,"mysql2dm [var_name]")(var_value)

	dm2mysql(var_n, var_value)
		if(!IsVariable(var_n)) return 0
		var/var_name = convert_char(var_n)

		if(!hascall(src,"dm2mysql [var_name]"))
			return var_value
		return call(src,"dm2mysql [var_name]")(var_value)

	InitInDB()
		if(!mysql)
			return 0

		var/DBQuery/Q = mysql.AddQuery()
		var/ok = Q.Execute(init_db_query)
		if(!ok)
			return 0

	Name()
		return "[tbl_name]"

	DM2MySQL(list/L)
		for(var/a in L)
			if(!(a in variables))
				continue
			L[a] = dm2mysql(a, L[a])
		return L

	MySQL2DM(list/L)
		for(var/a in L)
			if(!(a in variables))
				continue
			L[a] = mysql2dm(a, L[a])
		return L

	Update(list/L,condition,translate = 1)
		if(!L || !istype(L, /list)) return 0

		if(translate)
			DM2MySQL(L)
		else
			for(var/a in L)
				if(!(a in variables))
					L -= a
					continue

		var/DBQuery/Q = UpdateQuery(L,condition)
		if(istype(Q))
			mysql.RemoveQuery(Q)
			return 1
		return 0

	Insert(list/L,translate = 1,lock=0,fetch=0)
		if(!L || !istype(L, /list)) return 0
		if(translate)
			DM2MySQL(L)
		else
			for(var/a in L)
				if(!(a in variables))
					L -= a
					continue

		var/DBQuery/Q = InsertQuery(L,fetch)
		if(lock && istype(Q, /DBQuery))
			Q.locked = 1
			return Q
		else
			if(istype(Q, /DBQuery))
				return Q
			return 0

	Select(list/L,condition,lock = 0,limit)
		if(!L) return 0
		if(!istype(L, /list))
			L = list(L)

		for(var/a in L)
			if(!(a in variables) && a != "*")
				L -= a
				continue

		var/DBQuery/Q = SelectQuery(L,condition,limit)
		if(!istype(Q, /DBQuery))
			return Q
		if(lock) Q.locked = 1
		return Q

	Query2List(DBQuery/Query, translate = 1, first_only = 0)
		if(!Query) return
		var/list/l[0]

		for(var/i = 1, i <= Query.RowCount(), i++)
			Query.NextRow()
			l.len++
			var/list/data = Query.GetColumnData()
			if(translate)
				for(var/a in data)
					data[a] = mysql2dm(a, data[a])
			if(first_only) return data
			l[i] = data
		return l

	UpdateQuery(list/L,condition)
		var/sql = SQLFormat_Update(Name(),L,condition)
		if(!sql) return 0
		var/DBQuery/Q = mysql.AddQuery()
		if(!istype(Q))
			return Q
		Q.Execute(sql)
		return Q

	InsertQuery(list/L,fetch=0)
		var/sql = SQLFormat_Insert(Name(), L)
		if(!sql) return 0

		var/DBQuery/Q = mysql.AddQuery()
		if(!istype(Q))
			return Q
		Q.Execute(sql)
		if(fetch)
			Q.Execute("SELECT LAST_INSERT_ID() FROM [Name()]")
			Q.NextRow()
		return Q

	SelectQuery(list/L,condition,limit)
		var/sql = SQLFormat_Select(Name(),L,condition,limit)
		if(!sql) return 0

		var/DBQuery/Q = mysql.AddQuery()
		if(!istype(Q))
			return Q
		Q.Execute(sql)
		return Q
